@@ -4,7 +4,7 @@ use warnings;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '0.11';
+$VERSION = '0.14';
 
 #----------------------------------------------------------------------------
 
@@ -38,6 +38,15 @@ Validation of META.yml specification elements.
 #Specification Definitions													#
 #############################################################################
 
+my %known_specs = (
+    '1.4' => 'http://module-build.sourceforge.net/META-spec-v1.4.html',
+    '1.3' => 'http://module-build.sourceforge.net/META-spec-v1.3.html',
+    '1.2' => 'http://module-build.sourceforge.net/META-spec-v1.2.html',
+    '1.1' => 'http://module-build.sourceforge.net/META-spec-v1.1.html',
+    '1.0' => 'http://module-build.sourceforge.net/META-spec-v1.0.html'
+);
+my %known_urls = map {$known_specs{$_} => $_} keys %known_specs;
+
 my $module_map1 = { 'map' => { ':key' => { name => \&module, value => \&exversion } } };
 my $module_map2 = { 'map' => { ':key' => { name => \&module, value => \&version   } } };
 my $no_index_1_3 = {
@@ -60,6 +69,68 @@ my $no_index_1_1 = {
 };
 
 my %definitions = (
+'1.4' => {
+#  'header'              => { mandatory => 1, value => \&header },
+  'meta-spec'           => { mandatory => 1, 'map' => { version => { mandatory => 1, value => \&version},
+                                                        url     => { mandatory => 1, value => \&urlspec } } },
+
+  'name'                => { mandatory => 1, value => \&string  },
+  'version'             => { mandatory => 1, value => \&version },
+  'abstract'            => { mandatory => 1, value => \&string  },
+  'author'              => { mandatory => 1, list  => { value => \&string } },
+  'license'             => { mandatory => 1, value => \&license },
+  'generated_by'        => { mandatory => 1, value => \&string  },
+
+  'distribution_type'   => { value => \&string  },
+  'dynamic_config'      => { value => \&boolean },
+
+  'requires'            => $module_map1,
+  'recommends'          => $module_map1,
+  'build_requires'      => $module_map1,
+  'configure_requires'  => $module_map1,
+  'conflicts'           => $module_map2,
+
+  'optional_features'   => {
+    'map'       => {
+        ':key'  => { name => \&word,
+            'map'   => { description        => { value => \&string },
+                         requires_packages  => { value => \&string },
+                         requires_os        => { value => \&string },
+                         excludes_os        => { value => \&string },
+                         requires           => $module_map1,
+                         recommends         => $module_map1,
+                         build_requires     => $module_map1,
+                         conflicts          => $module_map2,
+            }
+        }
+     }
+  },
+
+  'provides'    => {
+    'map'       => { ':key' => { name  => \&module,
+                                 'map' => { file    => { mandatory => 1, value => \&file },
+                                            version => { value => \&version } } } }
+  },
+
+  'no_index'    => $no_index_1_3,
+  'private'     => $no_index_1_3,
+
+  'keywords'    => { list => { value => \&string } },
+
+  'resources'   => {
+    'map'       => { license    => { value => \&url },
+                     homepage   => { value => \&url },
+                     bugtracker => { value => \&url },
+                     repository => { value => \&url },
+                     ':key'     => { value => \&string, name => \&resource },
+    }
+  },
+
+  # additional user defined key/value pairs
+  # note we can only validate the key name, as the structure is user defined
+  ':key'        => { name => \&word },
+},
+
 '1.3' => {
 #  'header'              => { mandatory => 1, value => \&header },
   'meta-spec'           => { mandatory => 1, 'map' => { version => { mandatory => 1, value => \&version},
@@ -81,7 +152,7 @@ my %definitions = (
   'conflicts'           => $module_map2,
 
   'optional_features'   => {
-    list        => {
+    'map'       => {
         ':key'  => { name => \&word,
             'map'   => { description        => { value => \&string },
                          requires_packages  => { value => \&string },
@@ -150,6 +221,22 @@ my %definitions = (
   'build_requires'      => $module_map1,
   'conflicts'           => $module_map2,
 
+  'optional_features'   => {
+    'map'       => {
+        ':key'  => { name => \&word,
+            'map'   => { description        => { value => \&string },
+                         requires_packages  => { value => \&string },
+                         requires_os        => { value => \&string },
+                         excludes_os        => { value => \&string },
+                         requires           => $module_map1,
+                         recommends         => $module_map1,
+                         build_requires     => $module_map1,
+                         conflicts          => $module_map2,
+            }
+        }
+     }
+  },
+
   'provides'    => {
     'map'       => { ':key' => { name  => \&module,
                                  'map' => { file    => { mandatory => 1, value => \&file },
@@ -312,8 +399,13 @@ the appropriate specification definition.
 sub check_map {
     my ($self,$spec,$data) = @_;
 
+    if(ref($spec) ne 'HASH') {
+        $self->_error( "Unknown META.yml specification, cannot validate." );
+        return;
+    }
+
     if(ref($data) ne 'HASH') {
-        $self->_error( "Expected a map structure from YAML string or file" );
+        $self->_error( "Expected a map structure from YAML string or file." );
         return;
     }
 
@@ -427,7 +519,7 @@ Validates a list of versions, e.g. '<= 5, >=2, ==3, !=4, >1, <6, 0'.
 =item * version($self,$key,$value)
 
 Validates a single version string. Versions of the type '5.8.8' and '0.00_00'
-are both valid.
+are both valid. A leading 'v' like 'v1.2.3' is also valid.
 
 =item * boolean($self,$key,$value)
 
@@ -475,7 +567,7 @@ my $atom     = qr[a-z\d]i;
 my $domain   = qr((($atom(($atom|-)*$atom)?)\.)*([a-zA-Z](($atom|-)*$atom)?));
 my $ip       = qr((\d+)(\.(\d+)){3})(:(\d+))?;
 my $enc      = qr%[a-fA-F\d]{2};
-my $legal1   = qr[a-zA-Z\d\$\-_.+!*'(),]; #' - this comment is to avoid syntax highlighting issues
+my $legal1   = qr[a-zA-Z\d\$\-_.+!*'(),#]; #' - this comment is to avoid syntax highlighting issues
 my $legal2   = qr[;:@&=];
 my $legal3   = qr((($legal1|$enc)|$legal2)*);
 my $path     = qr\/$legal3(\/$legal3)*;
@@ -500,14 +592,6 @@ sub url {
     return 0;
 }
 
-my %known_specs = (
-    '1.3' => 'http://module-build.sourceforge.net/META-spec-v1.3.html',
-    '1.2' => 'http://module-build.sourceforge.net/META-spec-v1.2.html',
-    '1.1' => 'http://module-build.sourceforge.net/META-spec-v1.1.html',
-    '1.0' => 'http://module-build.sourceforge.net/META-spec-v1.0.html'
-);
-my %known_urls = map {$known_specs{$_} => $_} keys %known_specs;
-
 sub urlspec {
     my ($self,$key,$value) = @_;
     if(defined $value) {
@@ -561,7 +645,7 @@ sub version {
     my ($self,$key,$value) = @_;
     if(defined $value) {
         return 0    unless($value || $value =~ /0/);
-        return 1    if($value =~ /^\s*((<|<=|>=|>|!=|==)\s*)?\d+((\.\d+((_|\.)\d+)?)?)/);
+        return 1    if($value =~ /^\s*((<|<=|>=|>|!=|==)\s*)?v?\d+((\.\d+((_|\.)\d+)?)?)/);
     } else {
         $value = '<undef>';
     }
@@ -613,7 +697,9 @@ sub license {
 sub resource {
     my ($self,$key) = @_;
     if(defined $key) {
-        return 1    if($key && $key =~ /^([A-Z][a-z]+)+$/);
+        # a valid user defined key should be alphabetic 
+        # and contain at least one capital case letter.
+        return 1    if($key && $key =~ /^[a-z]+$/i && $key =~ /[A-Z]/);
     } else {
         $key = '<undef>';
     }
@@ -664,7 +750,7 @@ __END__
 There are no known bugs at the time of this release. However, if you spot a
 bug or are experiencing difficulties that are not explained within the POD
 documentation, please send an email to barbie@cpan.org or submit a bug to the
-RT system (http://rt.cpan.org/Public/Dist/Display.html?Name=Test-YAML-Meta).
+RT system (http://rt.cpan.org/Public/Dist/Display.html?Name=Test-CPAN-Meta).
 However, it would help greatly if you are able to pinpoint problems or even
 supply a patch.
 
@@ -686,7 +772,7 @@ for Miss Barbell Productions, L<http://www.missbarbell.co.uk>
 
 =head1 COPYRIGHT AND LICENSE
 
-  Copyright (C) 2007,2008 Barbie for Miss Barbell Productions
+  Copyright (C) 2007-2010 Barbie for Miss Barbell Productions
 
   This module is free software; you can redistribute it and/or
   modify it under the same terms as Perl itself.
